using System.ComponentModel;
using DataLayer;
using DataLayer.NorthwindDSTableAdapters;


namespace BusinessLayer
{
    [DataObject()]
    public class ProductsBL
    {

        private ProductsTableAdapter adapter = new ProductsTableAdapter();

        /// <summary>
        /// Default SELECT that gets all products
        /// </summary>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Select, true)]
        public NorthwindDS.ProductsDataTable GetProducts()
        {
            return adapter.GetData();
        }

        /// <summary>
        /// Gets products by the category id
        /// </summary>
        /// <param name="categoryId"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Select, false)]
        public NorthwindDS.ProductsDataTable GetProducts(int categoryId)
        {
            return adapter.GetDataByCategoryId(categoryId);
        }

        /// <summary>
        /// Gets the product for a specific productid
        /// </summary>
        /// <param name="productId"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Select, false)]
        public NorthwindDS.ProductsDataTable GetProduct(int productId)
        {
            return adapter.GetDataByProductId(productId);
        }

        /// <summary>
        /// Updates a product with supplied values
        /// </summary>
        /// <param name="ProductName"></param>
        /// <param name="SupplierID"></param>
        /// <param name="CategoryID"></param>
        /// <param name="QuantityPerUnit"></param>
        /// <param name="UnitPrice"></param>
        /// <param name="UnitsInStock"></param>
        /// <param name="UnitsOnOrder"></param>
        /// <param name="ReorderLevel"></param>
        /// <param name="Discontinued"></param>
        /// <param name="ProductID"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Update, true)]
        public bool Update(string ProductName, int? SupplierID, int? CategoryID, string QuantityPerUnit, decimal? UnitPrice,
             short? UnitsInStock, short? UnitsOnOrder, short? ReorderLevel, bool Discontinued, int ProductID)
        {

            NorthwindDS.ProductsDataTable products = adapter.GetDataByProductId(ProductID);
            if ((products.Count == 1))
            {
                NorthwindDS.ProductsRow row = products[0];
                row.ProductName = ProductName;
                if (SupplierID.HasValue) row.SupplierID = SupplierID.Value; else row.SetSupplierIDNull();
                if (CategoryID.HasValue) row.CategoryID = CategoryID.Value; else row.SetCategoryIDNull();
                row.QuantityPerUnit = QuantityPerUnit;
                if (UnitPrice.HasValue) row.UnitPrice = UnitPrice.Value; else row.SetUnitPriceNull();
                if (UnitsInStock.HasValue) row.UnitsInStock = UnitsInStock.Value; else row.SetUnitsInStockNull();
                if (UnitsOnOrder.HasValue) row.UnitsOnOrder = UnitsOnOrder.Value; else row.SetUnitsOnOrderNull();
                if (ReorderLevel.HasValue) row.ReorderLevel = ReorderLevel.Value; else row.SetReorderLevelNull();
                row.Discontinued = Discontinued;

                int result = adapter.Update(row);

                if (result == 1)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Adds a new product to the system
        /// </summary>
        /// <param name="ProductName"></param>
        /// <param name="SupplierID"></param>
        /// <param name="CategoryID"></param>
        /// <param name="QuantityPerUnit"></param>
        /// <param name="UnitPrice"></param>
        /// <param name="UnitsInStock"></param>
        /// <param name="UnitsOnOrder"></param>
        /// <param name="ReorderLevel"></param>
        /// <param name="Discontinued"></param>
        /// <param name="ProductID"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Insert, true)]
        public bool Add(string ProductName, int? SupplierID, int? CategoryID, string QuantityPerUnit, decimal? UnitPrice,
             short? UnitsInStock, short? UnitsOnOrder, short? ReorderLevel, bool Discontinued, int ProductID)
        {

            NorthwindDS.ProductsDataTable products = new NorthwindDS.ProductsDataTable();
            NorthwindDS.ProductsRow row = products.NewProductsRow();

            row.ProductName = ProductName;
            if (SupplierID.HasValue) row.SupplierID = SupplierID.Value; else row.SetSupplierIDNull();
            if (CategoryID.HasValue) row.CategoryID = CategoryID.Value; else row.SetCategoryIDNull();
            row.QuantityPerUnit = QuantityPerUnit;
            if (UnitPrice.HasValue) row.UnitPrice = UnitPrice.Value; else row.SetUnitPriceNull();
            if (UnitsInStock.HasValue) row.UnitsInStock = UnitsInStock.Value; else row.SetUnitsInStockNull();
            if (UnitsOnOrder.HasValue) row.UnitsOnOrder = UnitsOnOrder.Value; else row.SetUnitsOnOrderNull();
            if (ReorderLevel.HasValue) row.ReorderLevel = ReorderLevel.Value; else row.SetReorderLevelNull();
            row.Discontinued = Discontinued;

            int result = adapter.Update(row);
            if (result == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Gets the products but allows sorting and paging
        /// </summary>
        /// <param name="sortColumns"></param>
        /// <param name="startIndex"></param>
        /// <param name="maxRows"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        [DataObjectMethod(DataObjectMethodType.Select, false)]
        public NorthwindDS.ProductsDataTable GetProducts(string sortColumns, int startIndex, int maxRows)
        {
            return adapter.GetProducts(sortColumns, startIndex, maxRows);
        }

        /// <summary>
        /// Get a count of total products
        /// </summary>
        /// <returns></returns>
        /// <remarks></remarks>
        public int GetCount()
        {
            return adapter.GetCount().Value;
        }
    }


}